﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading.Tasks;
using System.Reflection;
using IronPython.Hosting;
using Microsoft.Scripting.Hosting;

namespace DSPython
{
    /* 引数による関数渡し用 */
    public delegate void TSendCommand(String inCommandText);


    static class CommonDef
    {
        public const String FWDCMD_LOCSPEED = "@SPD";
        public const String FWDCMD_LOCDIRECTION = "@DIR";
        public const String FWDCMD_LOCFUNCTION = "@FNC";
        public const String FWDCMD_TURNOUT = "@ACC";
        public const String FWDCMD_PING = "@PING";
        public const String FWDCMD_POWER = "@PWR";
        public const String FWDCMD_SETCV = "@CV";
        public const String FWDCMD_GETS88 = "@S88";

        public const int MM2ADDRESS = 0x0000;
        public const int MFXADDRESS = 0x4000;
        public const int DCCADDRESS = 0xC000;
        public const int MM2ACCADDRESS = 0x2FFF;
        public const int DCCACCADDRESS = 0x37FF;

    }

    public class DSScriptEngine
    {

        private bool Running = false;
        private ScriptEngine python;
        private ScriptScope scope;
        private ScriptRuntime runtime;
        private ScriptSource source;
        private CompiledCode code;
        private ScriptEngine engine;

        public DSScriptEngine(TSendCommand inFuncSend)
        {
            Assembly assembly = typeof(Program).Assembly;
            string dirPath = Path.GetDirectoryName(assembly.Location);
            string scriptPath = Path.Combine(dirPath, "Scripts");
            string filePath = Path.Combine(scriptPath, "main.py");

            python = Python.CreateEngine();

            runtime = python.Runtime;
            runtime.LoadAssembly(Assembly.LoadFile(assembly.Location));
            engine = Python.GetEngine(runtime);

            //実行エンジンに渡す値を設定する
            scope = python.CreateScope();

            //関数登録
            scope.SetVariable("DSGateway", new DSGatewayFunc(inFuncSend));

            //変数登録
            scope.SetVariable("MM2ADDRESS", CommonDef.MM2ADDRESS);
            scope.SetVariable("MFXADDRESS", CommonDef.MFXADDRESS);
            scope.SetVariable("DCCADDRESS", CommonDef.DCCADDRESS);
            scope.SetVariable("MM2ACCADDRESS", CommonDef.MM2ACCADDRESS);
            scope.SetVariable("DCCACCADDRESS", CommonDef.DCCACCADDRESS);

        }

        public void LoadFromScript(String inScriptText)
        {

            if (inScriptText == "")
            {
                return;
            }

            //Pythonのソースを指定
            source = engine.CreateScriptSourceFromString(inScriptText);
            code = source.Compile();


        }

        public void LoadFromFile(String inFileName)
        {
            
            if (File.Exists(inFileName) == false)
            {
                return;
            }
            
            //Pythonのソースを指定
            source = engine.CreateScriptSourceFromFile(inFileName);
            code = source.Compile();


        }

        public bool Start()
        {
            if (source == null)
            {

                return false;
            }

            Running = true;

            //ソースを実行する
            source.Execute(scope);

            return true;
        }

        public void Stop()
        {
            Running = false;

        }

        /* Pythonコード内の定周期タスクを実行 */
        public void RunTask(int inTaskLevel)
        {
            if( Running == false)
            {
                return;
            }

            switch (inTaskLevel)
            {
                case 0:
                    source.Execute(scope);
                    break;

                case 1:
                    python.Execute("loop()", scope);
                    break;

                case 2:
                    python.Execute("init()", scope);
                    break;

                default:
                    break;


            }



        }


        public bool IsRunning()
        {
            return Running;
        }
    }

    public class DSGatewayFunc
    {
        TSendCommand SendCommand;

        private String convertByteHex(int inValue)
        {
            int aAddressLow = inValue & 0xFF;
            int aAddressHigh = (inValue >> 8) & 0xFF;


            return aAddressHigh.ToString("x") + "," + aAddressLow.ToString("x");

        }

        public DSGatewayFunc(TSendCommand inFuncSend)
        {
            SendCommand = new TSendCommand(inFuncSend);

        }

        public void SetPower(int inPower)
        {
            SendCommand(CommonDef.FWDCMD_POWER + "," + inPower.ToString() + ",");
        }

        public void SetLocSpeed(int inAddress, int inSpeed)
        {
            SendCommand(CommonDef.FWDCMD_LOCSPEED + "," + convertByteHex(inAddress) + "," + convertByteHex(inSpeed) + ",");

        }

        public void SetLocoFunction(int inAddress, int inFunction, int inPower)
        {
            SendCommand(CommonDef.FWDCMD_LOCFUNCTION + "," + convertByteHex(inAddress) + "," + inFunction.ToString() + "," + inPower.ToString() + ",");
        }

        public void SetLocoDirection(int inAddress, int inDirection)
        {
            SendCommand(CommonDef.FWDCMD_LOCDIRECTION + "," + convertByteHex(inAddress) + "," + inDirection.ToString() + ",");
        }

        public void SetTurnout(int inAddress, int inSwitch)
        {
            SendCommand(CommonDef.FWDCMD_TURNOUT + "," + convertByteHex(inAddress) + "," + inSwitch.ToString() + ",");

        }

        public int GetS88(int address)
        {

            return 0;
        }

    }
}
